from ngsolve import *
from netgen.geom2d import unit_square

mesh = Mesh(unit_square.GenerateMesh(maxh=0.1))
#order = 1 -> H1 solution not good
order = 2
source = CoefficientFunction( (-y,x) )
eps = 1e-7

################### H1 ###################
fesH = VectorH1(mesh, order=order, dirichletx="top|bottom", dirichlety="left|right")
uH,vH = fesH.TnT()

Curl_u = CoefficientFunction( -Grad(uH)[0,1] + Grad(uH)[1,0] )
Curl_v = CoefficientFunction( -Grad(vH)[0,1] + Grad(vH)[1,0] )

aH = BilinearForm(fesH, symmetric=True)
aH += (Curl_u*Curl_v + eps*uH*vH)*dx
aH.Assemble()

fH = LinearForm(fesH)
fH += source*vH*dx
fH.Assemble()

gfuH = GridFunction(fesH)
gfuH.vec.data = aH.mat.Inverse(fesH.FreeDofs(), inverse="sparsecholesky")*fH.vec
Draw(Norm(gfuH), mesh, "uh")

################### HCurl ###################
fes = HCurl(mesh, order=order-1, dirichlet="top|bottom|left|right")
u,v = fes.TnT()

a = BilinearForm(fes, symmetric=True)
a += (curl(u)*curl(v) + eps*u*v)*dx
a.Assemble()

f = LinearForm(fes)
f += source*v*dx
f.Assemble()

gfu = GridFunction(fes)
gfu.vec.data = a.mat.Inverse(fes.FreeDofs(), inverse="sparsecholesky")*f.vec
Draw(Norm(gfu), mesh, "u")
